@@ -1,11 +1,6 @@
 package POE::Component::Server::IRC::Plugin;
-BEGIN {
-  $POE::Component::Server::IRC::Plugin::AUTHORITY = 'cpan:HINRIK';
-}
-{
-  $POE::Component::Server::IRC::Plugin::VERSION = '1.54';
-}
-
+our $AUTHORITY = 'cpan:BINGOS';
+$POE::Component::Server::IRC::Plugin::VERSION = '1.62';
 use strict;
 use warnings FATAL => 'all';
 
